using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using GrapeCity.Documents.Pdf;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Primitives;

namespace SubmitForm_Viewer.Controllers
{
    public class HandleFormSubmitXmlController : Controller
    {
        public IActionResult Index(IFormCollection fields)
        {
            //Parses the client response and builds the 'values' collection 
            //filling it with the submitted field values.  
            var fieldsTree = new List<FieldExportEntry>();
            foreach (var key in fields.Keys)
            {
                if (fields.TryGetValue(key, out StringValues value))

                {
                    var node = new FieldExportEntry() { Name = key };
                    foreach (var v in value)

                    { if (node.Values == null) node.Values = new List<string>(); node.Values.Add(v); }
                    fieldsTree.Add(node);
                }
            }

            ImportFormData(fieldsTree);            

            return View();
        }        

        // Creates a GcPdfDocument, loads an AcroForm PDF into it, and fills it with data
        // using the GcPdfDocument.ImportFormDataFromXML() method.      
        public static void ImportFormData(List<FieldExportEntry> values)
        {
            GcPdfDocument pdf = new GcPdfDocument();            
            using (var fs = new FileStream(Path.Combine("MyStaticFiles", "FormSubmitted.pdf"), FileMode.Open, FileAccess.Read))
            {
                pdf.Load(fs);
                using (var ms = new MemoryStream())
                {
                    SaveFieldsToXML(values, ms);                    
                    ms.Seek(0, SeekOrigin.Begin);
                    pdf.ImportFormDataFromXML(ms);
                }
                
                //Save the filled in form on the server
                pdf.Save("filledForm.pdf");               
            }
        }

        // Saves the fields and their values to a stream.       
        // This method is similar to GcPdfDocument.ExportFormDataToXML(), with the following
        // imporant limitations:
        // - it does not support child fields (field.Children collection);
        // - it does not handle fields with names that are not valid XML names (xfdf:original).
        public static void SaveFieldsToXML(List<FieldExportEntry> values, Stream stream)
        {                        
            XmlWriterSettings xws = new XmlWriterSettings()
            {
                Indent = true,
                CloseOutput = false,
                Encoding = Encoding.UTF8,
            };
            using (XmlWriter xw = XmlWriter.Create(stream, xws))
            {
                xw.WriteStartElement("fields");
                xw.WriteAttributeString("xmlns", "xfdf", null, "http://ns.adobe.com/xfdf-transition/");
                foreach (var ftn in values)
                {
                    xw.WriteStartElement(ftn.Name);
                    foreach (var v in ftn.Values)
                    {
                        xw.WriteStartElement("value");
                        // NOTE: the values in the array are formed by the client PDF viewer,
                        // and it represents 'on' checkbox values as 'true', while ImportFormDataFromXML
                        // expects 'on' values to be represented as "Yes" (that's how ExportFormDataToXML
                        // works, similar to Acrobat). This here is a quick hack for the same.                        
                        xw.WriteString(v == "true" ? "Yes" : v);
                        xw.WriteEndElement();
                    }
                    xw.WriteEndElement();
                }
                xw.WriteEndElement();
            }
        }
    }

    //Represents a form field and its value(s) for export to XML.
    public class FieldExportEntry
    {
        public string Name { get; set; }
        public List<string> Values { get; set; }
        // Note: this sample does not support child fields:
        // public List<FieldTreeNode> Children { get; set; }
    }
}
